# Generated by Django 4.2.8 on 2023-12-05 16:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0031_alter_orgbackend_index_together"),
    ]

    operations = [
        migrations.RenameIndex(
            model_name="orgbackend",
            new_name="orgs_orgbac_org_id_607508_idx",
            old_fields=("org", "is_active", "slug"),
        ),
    ]
