# Generated by Django 4.1.7 on 2023-03-29 14:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("orgs", "0031_alter_orgbackend_index_together"),
        ("dashblocks", "0010_alter_dashblock_created_by_and_more"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="dashblock",
            index=models.Index(
                fields=["org", "is_active", "dashblock_type", "priority"], name="dashblocks__org_id_024805_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="dashblocktype",
            index=models.Index(fields=["slug", "name"], name="dashblocks__slug_c0c6c6_idx"),
        ),
    ]
