# Generated by Django 5.0.8 on 2024-08-21 16:03

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("orgs", "0032_rename_orgbackend_org_is_active_slug_orgs_orgbac_org_id_607508_idx"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameIndex(
            model_name="orgbackend",
            new_name="orgs_orgbac_org_slug_idx",
            old_name="orgs_orgbac_org_id_607508_idx",
        ),
        migrations.AlterUniqueTogether(
            name="orgbackend",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="taskstate",
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name="orgbackend",
            constraint=models.UniqueConstraint(fields=("org", "slug"), name="orgs_orgbackend_org_slug_unique"),
        ),
        migrations.AddConstraint(
            model_name="taskstate",
            constraint=models.UniqueConstraint(fields=("org", "task_key"), name="orgs_taskstate_org_task_key_unique"),
        ),
    ]
